#!/usr/bin/perl

###############################################################################
#                                                                             #
#               Dimke WebMon VNVM 1.0                                         #
#                                                                             #
#               All rights reserved. Copyright (C) 2008 - 2009                #
#               Dipl.-Ing. Markus Robert Keler                               #
#               Bad Schussenried, Germany                                     #
#                                                                             #
#               For latest news see http://www.dipl-ing-kessler.de/developer  #
#                                                                             #
###############################################################################
#                                                                             #
#               Purpose:                                                      #
#                                                                             #
#               Dimke WebMon monitors a given list of sites located on        #
#               distant webservers by requesting information about them       #
#               either via HTTP HEAD for static web pages or via              #
#               HTTP GET needed for dynamic web pages and send out            #
#               information about their status via email.                     #
#               If there is no access to a document, Dimke WebMon VNVM        #
#               will send an email notification, also Dimke WebMon will       #
#               inform you about as soon as the monitored server is up and    #
#               running again. Also Dimke WebMon can be configured to send    #
#               an email notification if the document had been changed        #
#               since the last access.                                        #
#                                                                             #
###############################################################################
#                                                                             #
#               The following cases may occur and have to be handled:         #
#                                                                             #
#                          Fetch_ok_act  Fetch_ok_last  Timestamp_changed     #
#                                                                             #
#               Case_1:            0             0               0            #
#                     ==> No action                                           #
#               Case_2:            0             0               1            #
#                     ==> No action                                           #
#               Case_3:            0             1               0            #
#                     ==> Email "Server down"; Write status                   #
#               Case_4:            0             1               1            #
#                     ==> Email "Server down"; Write status                   #
#               Case_5:            1             0               0            #
#                     ==> Email "Server up again"; Write status               #
#               Case_6:            1             0               1            #
#                     ==> Email "Server up again, File changed"; Write status #
#               Case_7:            1             1               0            #
#                     ==> No action                                           #
#               Case_8:            1             1               1            #
#                     ==> Email "File changed"; Write status                  #
#                                                                             #
###############################################################################


$Path_to_webmon  = '/path/to/webmonu/';


###############################################################################
###############################################################################
#                                                                             #
#   ! ! !     Cave, lector: Never change any parameter below here     ! ! !   #
#                                                                             #
###############################################################################
###############################################################################

require            "$Path_to_webmon/webmonu.cfg";

foreach $current_site ( @list_of_sites ) {

    chomp ( $current_site );
    require "$Path_to_webmon/$current_site";

    $timestamp       = `date +'%d.%m.%Y %H:%M:%S'`; chomp ( $timestamp );
    $message         = '';

    use LWP::UserAgent;

    if ( $http_type eq 'HEAD' ) {
        $header = new HTTP::Headers;
        $header->user_agent($agent);
        $ua = new LWP::UserAgent;
        $req = new HTTP::Request ("HEAD" => $ref_file, $header);
        $result = $ua->request($req);
        $modified_new = $result->last_modified;
    }
    else {
        $ua = new LWP::UserAgent;
        $ua->agent($agent);
        $req = new HTTP::Request 'GET', $ref_file;
        $req-> header('Accept' => 'text/html');
        $result_new = $ua->request($req)->content;
        use Digest::MD5 qw(md5_hex);
        $modified_new = md5_hex ( $result_new );
    }


    #   ### Load old header information and access result #########################

    open HEADINFO, "$Path_to_webmon/$headinfo_file" or die "Couldn't open $headinfo_file: $!\n";
        $modified_old = <HEADINFO>; chomp ( $modified_old );
        $Fetch_ok_last = <HEADINFO>; chomp ( $Fetch_ok_last );
    close HEADINFO;
    if ( $Fetch_ok_last ne 'yes' ) { $Fetch_ok_last = 'no'; }


    $Fetch_ok_act = ( $modified_new eq '' ? 'no' : 'yes' );

    if ( $modified_new eq $modified_old ) {
        $Timestamp_changed = 'no';
    }
    elsif ( $Fetch_ok_act eq 'yes' ) {
        $Timestamp_changed = 'yes';
    }
    else {
        $Timestamp_changed = 'NA';
    }




    #   ### Case_1,2 ##############################################################

    if ( $Fetch_ok_act eq 'no' && $Fetch_ok_last eq 'no' ) {
    }

    #   ### Case_3,4 ##############################################################

    elsif ( $Fetch_ok_act eq 'no' && $Fetch_ok_last eq 'yes' ) {
        Send_Email ( "Server nicht erreichbar" );
        $modified_new = $modified_old;
        Swrite_headerfile ( );
    }

    #   ### Case_5 ################################################################

    elsif ( $Fetch_ok_act eq 'yes' && $Fetch_ok_last eq 'no' && $Timestamp_changed eq 'no' ) {
        Send_Email ( "Server wieder erreichbar\n\nReferenzdatei nicht gendert" );
        Swrite_headerfile ( );
    }

    #   ### Case_6 ################################################################

    elsif ( $Fetch_ok_act eq 'yes' && $Fetch_ok_last eq 'no' && $Timestamp_changed eq 'yes' ) {
        Send_Email ( "Server wieder erreichbar\n\nReferenzdatei gendert: $ref_file" );
        Swrite_headerfile ( );
    }

    #   ### Case_7 ################################################################

    elsif ( $Fetch_ok_act eq 'yes' && $Fetch_ok_last eq 'yes' && $Timestamp_changed eq 'no' ) {
    }

    #   ### Case_8 ################################################################

    elsif ( $Fetch_ok_act eq 'yes' && $Fetch_ok_last eq 'yes' && $Timestamp_changed eq 'yes' ) {
        Send_Email ( "Referenzdatei gendert: $ref_file" );
        Swrite_headerfile ( );
    }


    if ( $write_log_file eq 'yes' ) {
        Swrite_logfile ( );
    }





    sub Send_Email {

        my ( $email_body ) = @_;
        open(EmailFile,"$Path_to_webmon/$to_file") or die "Couldn't open $to_file: $!\n";
            @emails=<EmailFile>;
        close(EmailFile);
        $ToField='';
        foreach $i (@emails) {
            chomp($i);
            $ToField .= "$i,";
        }
        close(EmailFile);

        open(EmailFile,"$Path_to_webmon/$bcc_file") or die "Couldn't open $bcc_file: $!\n";
            @emails=<EmailFile>;
        close(EmailFile);
        $BccField='';
        foreach $i (@emails) {
            chomp($i);
            $BccField .= "$i,";
        }
        close(EmailFile);

        open (MAIL, "|$mailprog -t") || die "Can't open $mailprog!\n";
        print MAIL "From: $from\n";
        print MAIL "Reply-to: $from\n";
        print MAIL "To: $ToField\n";
        print MAIL "Bcc: $BccField\n";
        print MAIL "X-Mailer: $x_mailer\n";
        print MAIL "X-Priority: $priority\n";
        print MAIL "Subject: $subject\n";
        print MAIL "MIME-Version: 1.0\n";
        print MAIL "Content-Type: text/plain; charset=\"iso-8859-1\"\n\n";

        print MAIL "################################################################################\n";
        print MAIL "\n";
        print MAIL "\t\t\tDimke WebMon - Serverstatus\n";
        print MAIL "\n";
        print MAIL "################################################################################\n";
        print MAIL "\n\n";
        print MAIL "$email_body";
        print MAIL "\n";

        close (MAIL);

    }


    sub Swrite_headerfile {
        open HEADINFO, ">$Path_to_webmon/$headinfo_file" or die "Couldn't open $headinfo_file: $!\n";
            print HEADINFO "$modified_new\n";
            print HEADINFO "$Fetch_ok_act\n";
        close HEADINFO;
    }


    sub Swrite_logfile {
        my $logfile_exists =  ( ( -e "$Path_to_webmon/$log_file" ) ? 'yes' : 'no' );
        open LOGFILE, ">>$Path_to_webmon/$log_file" or die "Couldn't open $log_file: $!\n";
            if ( $logfile_exists eq 'no' ) {

                print LOGFILE "$agent\n\nAccess statistics\n\n";
                print LOGFILE "Timestamp_Date_Time\tAcc_OK\tFile_chgd\tSite\n\n";
            }
            print LOGFILE "$timestamp\t$Fetch_ok_act\t$Timestamp_changed\t\t$ref_file\n";
        close HEADINFO;
    }

}

exit;
